from searchunify.client.client import CONSTANTS
from searchunify.utils.constants.constants import AuthVariables, Constants

AUTH_VARS = AuthVariables()
CONSTANTS = Constants()

def get_auth_header():
    authHeader = { 
        'Authorization': None,
        'Content-Type': 'application/json'
    }
    if(AUTH_VARS.AUTH_TYPE == CONSTANTS.OAUTH2_DEL):
        authHeader['Authorization'] = 'Bearer ' + AUTH_VARS.ACCESS_TOKEN
    elif(AUTH_VARS.AUTH_TYPE == CONSTANTS.API_KEY_DEL):
        authHeader['Authorization'] = 'ApiKey ' + AUTH_VARS.ACCESS_TOKEN
    elif(AUTH_VARS.AUTH_TYPE == CONSTANTS.JWT_KEY_DEL):
        authHeader['Authorization'] = 'Jwt ' + AUTH_VARS.JWT_TOKEN
    else:
        raise Exception('Invalid authorization header.')
    
    return authHeader



from searchunify.client.client import CONSTANTS
from searchunify.utils.http_client.request_handler import RequestManager
from searchunify.utils.apis.apis import Apis
from searchunify.utils.constants.constants import AuthVariables, Constants
import base64

API = Apis.get_apis('oauth')
AUTH_VARS = AuthVariables()
CONSTANTS = Constants()

def base64_encode(args):
    message = args.get('client_id')+":"+args.get('client_secret')
    message = message.encode('ascii')
    message = base64.b64encode(message)
    base64_message = message.decode('ascii')
    playLoad = 'grant_type='+CONSTANTS.GRANT_TYPE+'&username='+args.get('username')+'&password='+args.get('password')
    headers = {
        'Content-Type': 'application/x-www-form-urlencoded',
        'Authorization': 'Basic '+base64_message,
        'cache-contro': 'no-cache'
    }
    return {"playload": playLoad, "headers": headers}

class Oauth2:
    def generate_access_token(args):
        try:
            url = AUTH_VARS.INSTANCE + API['accessToken']
            del args['instance']
            result = base64_encode(args)
            response = RequestManager.http_request(
                "POST", url, headers=result["headers"], data=result["playload"])
            if (response['accessToken'] and response['refreshToken']):
                AuthVariables.ACCESS_TOKEN = response['accessToken']
                AuthVariables.REFRESH_TOKEN = response['refreshToken']
                return 'SearchUnify client initialize successfully.'
            else:
                raise Exception('SearchUnify client not initialized.')
        except Exception as e:
            raise Exception('SearchUnify client not initialized.', e)


    def access_token_from_refresh_token():
        try:
            url = AUTH_VARS.INSTANCE + API['accessToken']
            headers = {
                'Content-Type': 'application/x-www-form-urlencoded',
                'cache-contro': 'no-cache'
            }
            payload = 'grant_type='+CONSTANTS.GRANT_TYPE_REFRESH_TOKEN+'&refresh_token='+AUTH_VARS.REFRESH_TOKEN + '&client_id='+ AUTH_VARS.CLIENT_ID + '&client_secret='+ AUTH_VARS.SECRETS
            response = RequestManager.http_request(
                "POST", url, headers=headers, data=payload)
            if (response['accessToken'] and response['refreshToken']):
                AuthVariables.ACCESS_TOKEN = response['accessToken']
                AuthVariables.REFRESH_TOKEN = response['refreshToken']
                return 'Token refreshed successfully.'
            else:
                raise Exception('Token not refreshed.')
        except Exception as e:
            raise Exception('Access token not refreshed.', e)



from schema import Schema, And, Use, Optional

class ContentValidator:
    def content_source_id_validation(**args):
        schema = {
            'contentSourceId': And(Use(str), lambda s: len(s) > 0, error="contentSourceId is required.")
        }
        Schema(schema).validate(args)

    def get_object_specific_data(**args):
        schema = {
            'contentSourceId': And(Use(str), lambda s: len(s) > 0, error="contentSourceId is required."),
            'objectId': And(Use(str), lambda s: len(s) > 0, error="objectId is required."),
            Optional('offset'): And(Use(int), lambda n: n >= 0, error="offset value should be greater than or equal to 0."),
            Optional('size'): int,
        }
        Schema(schema).validate(args)

    def get_object_specific_data_with_id(**args):
        schema = {
            'contentSourceId': And(Use(str), lambda s: len(s) > 0, error="contentSourceId is required."),
            'objectId': And(Use(str), lambda s: len(s) > 0, error="objectId is required."),
            'documentId': And(Use(str), lambda s: len(s) > 0, error="documentId is required.")
        }
        Schema(schema).validate(args)

    def update_doucment_by_id(**args):
        schema = {
            'contentSourceId': And(Use(str), lambda s: len(s) > 0, error="contentSourceId is required."),
            'objectId': And(Use(str), lambda s: len(s) > 0, error="objectId is required."),
            'documentId': And(Use(str), lambda s: len(s) > 0, error="documentId is required."),
            'data': dict
        }
        Schema(schema).validate(args)

    def upload_data(**args):
        schema = {
            'contentSourceId': And(Use(str), lambda s: len(s) > 0, error="contentSourceId is required."),
            'objectId': And(Use(str), lambda s: len(s) > 0, error="objectId is required."),
            'data': list
        }
        Schema(schema).validate(args)



su_apis = {
    "analytics": {
        "tileData": "/api/v2/searchOverview/getTileData",
        "searchSummaryChart": "/api/v2/searchOverview/getSearchSummaryChart",
        "searchQueryAll": "/api/v2/searchQuery/all",
        "searchQueryWithResults": "/api/v2/searchQuery/withResults",
        "searchQueryWithNoClicks": "/api/v2/searchQuery/withNoClicks",
        "searchQueryWithoutResult": "/api/v2/searchQuery/withoutResults",
        "searchQueryHistogram": "/api/v2/searchQuery/histogram",
        "missedQueryHistogram": "/api/v2/searchQuery/missedQueryHistogram",
        "searchSessionByCaseUidAuth": "/api/v2/searchSession/byCaseUidAuth",
        "allSearchConversion": "/api/v2/searchConversion/all",
        "searchConversionNotOnFirstPage": "/api/v2/searchConversion/notOnFirstPage",
        "searchConversionWithFilters": "/api/v2/searchConversion/withFilters",
        "searchConversionBySessionId": "/api/v2/searchConversion/bySessionsId/<searchSessionId>",
        "discussionsReadyToBecomeArticles": "/api/v2/searchConversion/DiscussionsReadyToBecomeArticles",
        "searchSessionBySearchSessionId": "/api/v2/searchSession/bySearchSessionId/<searchSessionId>",
        "searchSessionAllsearchQuery": "/api/v2/searchSession/all/searchQuery",
        "kcsSupport": "/api/v2/searchQuery/kcsSupport",
        "searchSessionByCaseUid": "/api/v2/searchSession/byCaseUid",
        "articlesCreatedCases": "/api/v2/conversion/articlesCreatedCases",
        "articlesDeflectedCase": "/api/v2/conversion/articlesDeflectedCase",
        "attachedArticles": "/api/v2/conversion/attachedArticles",
        "attachedOnCase": "/api/v2/conversion/attachedOnCase"
    },
    "search": {
        "searchResults": "/api/v2_search/searchResults"
    },
    "content": {
        "contentSources": "/api/v2_cs/contentSource/all",
        "contentSourceByID": "/api/v2_cs/contentSource",
        "objectAndFields": "/api/v2_cs/contentSource/<contentSourceId>/objectAndFields",
        "objectData": "/api/v2_cs/apiData/contentSource/<contentSourceId>/object/<objectId>/get",
        "objectDataWithId": "/api/v2_cs/apiData/contentSource/<contentSourceId>/object/<objectId>/document/<documentId>/get",
        "updateDoucmentById": "/api/v2_cs/apiData/contentSource/<contentSourceId>/object/<objectId>/document/<documentId>/update",
        "bulkUpload": "/api/v2_cs/apiData/contentSource/<contentSourceId>/object/<objectId>/bulkUpload"
    },
    "oauth": {
        "accessToken": "/oauth/token"
    }
}

class Apis:
    def get_apis(type):
        if (type):
            return su_apis[type]
        else:
            return 'Unknown api type.'



from searchunify.utils.http_client.request_handler import RequestManager
from searchunify.core.auth.auth_header_type import get_auth_header
from searchunify.utils.constants.constants import AuthVariables
from searchunify.utils.apis.apis import Apis
import urllib.parse
import re
import json

API = Apis.get_apis('content')
AUTH_VARS = AuthVariables()

def replace_all(url, replacedObject):
    rep = dict((re.escape(k), urllib.parse.quote_plus(urllib.parse.quote_plus(v))) for k, v in replacedObject.items())
    pattern = re.compile("|".join(rep.keys()))
    url = pattern.sub(lambda m: rep[re.escape(m.group(0))], url)
    return url

class Content:
    def get_content_sources():
        url = AUTH_VARS.INSTANCE + API['contentSources']
        response = RequestManager.http_request(
            "GET", url, headers=get_auth_header())
        return response


    def get_content_source_by_id(**payload):
        url = AUTH_VARS.INSTANCE + \
            API['contentSourceByID'] + "/" + str(payload['contentSourceId'])
        response = RequestManager.http_request(
            "GET", url, headers=get_auth_header())
        return response


    def get_object_and_fields(**payload):
        url = AUTH_VARS.INSTANCE + API['objectAndFields']
        url = url.replace("<contentSourceId>", str(payload['contentSourceId']), 1)
        response = RequestManager.http_request(
            "GET", url, headers=get_auth_header())
        return response


    def get_object_specific_data(**payload):
        url = AUTH_VARS.INSTANCE + API['objectData']
        replacedObject = {"<contentSourceId>": str(
            payload['contentSourceId']), "<objectId>": str(payload['objectId'])}
        url = replace_all(url, replacedObject)
        payload["from"] = payload["offset"]
        del payload["contentSourceId"], payload["objectId"], payload["offset"]
        response = RequestManager.http_request(
            "GET", url, headers=get_auth_header(), params=payload)
        return response


    def get_object_specific_data_with_id(**payload):
        url = AUTH_VARS.INSTANCE + API['objectDataWithId']
        replacedObject = {"<contentSourceId>": str(payload['contentSourceId']), "<objectId>": str(
            payload['objectId']), "<documentId>": str(payload['documentId'])}
        url = replace_all(url, replacedObject)
        response = RequestManager.http_request(
            "GET", url, headers=get_auth_header(), params=payload)
        return response


    def update_doucment_by_id(**payload):
        url = AUTH_VARS.INSTANCE + API['updateDoucmentById']
        replacedObject = {"<contentSourceId>": str(payload['contentSourceId']), "<objectId>": str(
            payload['objectId']), "<documentId>": str(payload['documentId'])}
        url = replace_all(url, replacedObject)
        payload = json.dumps(payload['data'])
        response = RequestManager.http_request(
            "POST", url, headers=get_auth_header(), data=payload)
        return response


    def upload_data(**payload):
        url = AUTH_VARS.INSTANCE + API['bulkUpload']
        replacedObject = {"<contentSourceId>": str(
            payload['contentSourceId']), "<objectId>": str(payload['objectId'])}
        url = replace_all(url, replacedObject)
        payload = {
            "bulkData": payload['data']
        }
        payload = json.dumps(payload)
        response = RequestManager.http_request(
            "POST", url, headers=get_auth_header(), data=payload)
        return response



import requests
from requests.exceptions import HTTPError
import json


class RequestManager:
    def http_request(method, url, **args):
        try:
            result = requests.request(method, url, **args)
            response = json.loads(result.text)
            return response
        except HTTPError as httpError:
            raise Exception(httpError)
        except Exception as error:
            raise Exception(error)



#!/usr/bin/env python

import setuptools

setuptools.setup(
    include_package_data=True,
    name="searchunify",
    version='1.0.3',
    description='The SearchUnify SDK enables developers to easily work with the SearchUnify platform and build scalable solutions with search, analytics, crawlers and more.',
    author='SearchUnify',
    author_email='ankur.mahajan@grazitti.com',
    url='https://www.searchunify.com/',
    packages=[
        'searchunify',
        'searchunify.core',
        'searchunify.core.analytics',
        'searchunify.core.auth',
        'searchunify.core.content',
        'searchunify.core.search',
        'searchunify.client', 
        'searchunify.utils',
        'searchunify.utils.apis',
        'searchunify.utils.constants',
        'searchunify.utils.http_client',
        'searchunify.utils.validation'
    ],
    install_requires=['requests==2.23.0', 'schema==0.7.4']
)


from schema import Schema, And, Use, Regex

class SearchValidator:
    def get_search_results(**args):
        schema = {
            'searchString': str,
            'uid': And(Use(str), Regex(r'^[0-9a-f]{8}-[0-9a-f]{4}-[0-9a-f]{4}-[0-9a-f]{4}-[0-9a-f]{12}$'), error="Invalid uuid format for uid.")
        }
        Schema(schema, ignore_extra_keys=True).validate(args)



from searchunify.utils.constants.constants import AuthVariables
from searchunify.core.auth.auth_header_type import get_auth_header
from searchunify.utils.http_client.request_handler import *
from searchunify.utils.apis.apis import Apis
import json

API = Apis.get_apis('analytics')
AUTH_VARS = AuthVariables()

class Analytics:
    def get_tiles_data(**payload):
        url = AUTH_VARS.INSTANCE + API['tileData']
        payload = json.dumps(payload)
        response = RequestManager.http_request(
            "POST", url, headers=get_auth_header(), data=payload)
        return response

    def get_search_summary_chart(**payload):
        url = AUTH_VARS.INSTANCE + API['searchSummaryChart']
        payload = json.dumps(payload)
        response = RequestManager.http_request(
            "POST", url, headers=get_auth_header(), data=payload)
        return response


    def get_all_search_query(**payload):
        url = AUTH_VARS.INSTANCE + API['searchQueryAll']
        response = RequestManager.http_request(
            "GET", url, headers=get_auth_header(), params=payload)
        return response


    def get_search_queries_with_result(**payload):
        url = AUTH_VARS.INSTANCE + API['searchQueryWithResults']
        response = RequestManager.http_request(
            "GET", url, headers=get_auth_header(), params=payload)
        return response

    def search_queries_with_no_clicks(**payload):
        url = AUTH_VARS.INSTANCE + API['searchQueryWithNoClicks']
        response = RequestManager.http_request(
            "GET", url, headers=get_auth_header(), params=payload)
        return response


    def search_queries_without_result(**payload):
        url = AUTH_VARS.INSTANCE + API['searchQueryWithoutResult']
        response = RequestManager.http_request(
            "GET", url, headers=get_auth_header(), params=payload)
        return response


    def search_query_histogram(**payload):
        url = AUTH_VARS.INSTANCE + API['searchQueryHistogram']
        payload = json.dumps(payload)
        response = RequestManager.http_request(
            "POST", url, headers=get_auth_header(), data=payload)
        return response


    def missed_query_histogram(**payload):
        url = AUTH_VARS.INSTANCE + API['missedQueryHistogram']
        payload = json.dumps(payload)
        response = RequestManager.http_request(
            "POST", url, headers=get_auth_header(), data=payload)
        return response


    def search_session_by_case_uid_auth(**payload):
        url = AUTH_VARS.INSTANCE + API['searchSessionByCaseUidAuth']
        payload = json.dumps(payload)
        response = RequestManager.http_request(
            "POST", url, headers=get_auth_header(), data=payload)
        return response


    def get_all_search_conversion(**payload):
        url = AUTH_VARS.INSTANCE + API['allSearchConversion']
        response = RequestManager.http_request(
            "GET", url, headers=get_auth_header(), params=payload)
        return response


    def search_conversion_not_On_first_page(**payload):
        url = AUTH_VARS.INSTANCE + API['searchConversionNotOnFirstPage']
        response = RequestManager.http_request(
            "GET", url, headers=get_auth_header(), params=payload)
        return response


    def search_conversion_with_filters(**payload):
        url = AUTH_VARS.INSTANCE + API['searchConversionWithFilters']
        response = RequestManager.http_request(
            "GET", url, headers=get_auth_header(), params=payload)
        return response


    def search_conversion_by_sessionid(**payload):
        url = AUTH_VARS.INSTANCE + API['searchConversionBySessionId']
        url = url.replace("<searchSessionId>", str(payload['sessionId']), 1)
        response = RequestManager.http_request(
            "GET", url, headers=get_auth_header(), params=payload)
        return response


    def discussions_ready_to_become_article(**payload):
        url = AUTH_VARS.INSTANCE + \
            API['discussionsReadyToBecomeArticles']
        payload = json.dumps(payload)
        response = RequestManager.http_request(
            "POST", url, headers=get_auth_header(), data=payload)
        return response


    def search_queries_in_sessions(**payload):
        url = AUTH_VARS.INSTANCE + API['searchSessionAllsearchQuery']
        response = RequestManager.http_request(
            "GET", url, headers=get_auth_header(), params=payload)
        return response


    def get_kcs_support_search_query(**payload):
        url = AUTH_VARS.INSTANCE + API['kcsSupport']
        payload = json.dumps(payload)
        response = RequestManager.http_request(
            "POST", url, headers=get_auth_header(), data=payload)
        return response


    def get_search_session_by_case_uid(**payload):
        url = AUTH_VARS.INSTANCE + API['searchSessionByCaseUid']
        payload["caseUid"] = payload['caseUID']
        del payload['caseUID']
        payload = json.dumps(payload)
        response = RequestManager.http_request(
            "POST", url, headers=get_auth_header(), data=payload)
        return response


    def get_search_session_by_search_sessionid(**payload):
        url = AUTH_VARS.INSTANCE + API['searchSessionBySearchSessionId']
        url = url.replace("<searchSessionId>", str(payload['sessionId']), 1)
        response = RequestManager.http_request(
            "GET", url, headers=get_auth_header(), params=payload)
        return response


    def get_case_created_articles(**payload):
        url = AUTH_VARS.INSTANCE + API['articlesCreatedCases']
        payload = re_modify_keys(**payload)
        payload = json.dumps(payload)
        response = RequestManager.http_request(
            "POST", url, headers=get_auth_header(), data=payload)
        return response


    def get_case_deflected_articles(**payload):
        url = AUTH_VARS.INSTANCE + API['articlesDeflectedCase']
        payload = re_modify_keys(**payload)
        payload = json.dumps(payload)
        response = RequestManager.http_request(
            "POST", url, headers=get_auth_header(), data=payload)
        return response


    def get_attached_articles(**payload):
        url = AUTH_VARS.INSTANCE + API['attachedArticles']
        payload = re_modify_keys(**payload)
        payload = json.dumps(payload)
        response = RequestManager.http_request(
            "POST", url, headers=get_auth_header(), data=payload)
        return response


    def get_attached_on_case(**payload):
        url = AUTH_VARS.INSTANCE + API['attachedOnCase']
        payload = re_modify_keys(**payload)
        payload = json.dumps(payload)
        response = RequestManager.http_request(
            "POST", url, headers=get_auth_header(), data=payload)
        return response

def re_modify_keys(**payload):
    payload["from"] = payload['startDate']
    payload["to"] = payload['endDate']
    payload["uid"] = payload['searchClientId']
    payload["limit"] = payload['count']
    del payload['startDate'],  payload['endDate'], payload['searchClientId'], payload['count']
    return payload

